import fileinput
import os
import shutil

import docker
from dotenv import dotenv_values

from magniv.utils.utils import _create_cloud_build


def export_to_airflow(
    task_list,
    gcp=False,
    gcp_project_id=None,
    gcp_dag_folder=None,
    callback_hook=None,
    env_file_path=None,
):
    dag_template_filename = "dag-template.py"
    dag_template_directory = "{}/{}".format(
        os.path.dirname(__file__), dag_template_filename
    )
    docker_image_info = []
    for task_info in task_list:
        print("starting task .... ")
        new_filename = "dags/{}/{}.py".format(task_info["owner"], task_info["key"])
        print(new_filename)
        if not os.path.exists("dags/"):
            os.mkdir("dags")
        if not os.path.exists("dags/{}/".format(task_info["owner"])):
            os.mkdir("dags/{}/".format(task_info["owner"]))
        shutil.copyfile(dag_template_directory, new_filename)
        print("creating docker image ... ")
        docker_name, path = _create_docker_image(
            task_info["python_version"],
            task_info["requirements_location"],
            task_info["key"],
            gcp=gcp,
            gcp_project_id=gcp_project_id,
            env_file_path=env_file_path,
        )
        print("docker image created!")
        docker_image_info.append((docker_name, path))
        print("creating dag ... ")
        with fileinput.input(new_filename, inplace=True) as f:
            for line in f:
                line = (
                    line.replace("dag_id", "'{}'".format(task_info["key"]))
                    .replace("ownertoreplace", "'{}'".format(task_info["owner"]))
                    .replace("scheduletoreplace", "'{}'".format(task_info["schedule"]))
                    .replace("imagetoreplace", "'{}'".format(docker_name))
                    .replace("filetoreplace", task_info["location"])
                    .replace("functiontoreplace", task_info["name"])
                    .replace(
                        "callbackhooktoreplace",
                        "'{}'".format(callback_hook)
                        if callback_hook != None
                        else "None",
                    )
                    .replace(
                        "successtoreplace",
                        "_on_success" if callback_hook != None else "None",
                    )
                    .replace(
                        "failuretoreplace",
                        "_on_failure" if callback_hook != None else "None",
                    )
                )
                print(line, end="")
        print("dag created!")
    if gcp:
        _create_cloud_build(docker_image_info, gcp_dag_folder)


def _create_docker_image(
    python_version,
    requirements,
    key,
    gcp=False,
    gcp_project_id=None,
    env_file_path=None,
):
    path = "/".join(requirements.split("/")[:-1])
    if not gcp:
        requirements = "requirements.txt"
    environment_arguments = ''
    if env_file_path != None:
        env_values_dict = dotenv_values(env_file_path)
        environment_arguments = "\n".join(
            ["ENV {}={}".format(key, env_values_dict[key]) for key in env_values_dict]
        )
    dockerfile = """
# syntax=docker/dockerfile:1

FROM python:{}
COPY {} requirements.txt
{}
RUN pip3 install -r requirements.txt

COPY . .
                """.format(
        python_version, requirements, environment_arguments
    )
    with open("{}/Dockerfile".format(path), "w") as fo:
        fo.write(dockerfile)
    docker_name = "{}dockerimage".format(key)
    if gcp:
        docker_name = "gcr.io/{}/{}".format(gcp_project_id, docker_name)
    else:
        client = docker.from_env()
        client.images.build(path=path, tag=docker_name)
    return docker_name, path



import json


def _save_to_json(obj, filepath):
    with open(filepath, "w") as fo:
        json.dump(obj, fo)


def _get_tasks_json(filepath):
    with open(filepath) as fo:
        task_list = json.load(fo)
    return task_list


def _get_function_from_json(key, filepath):
    f = open(filepath)
    task_info_list = json.load(f)
    for task in task_info_list:
        if task["key"] == key:
            break
    return task["location"], task["name"]


def _create_cloud_build(docker_image_info, gcp_dag_folder):
    with open("./cloudbuild.yaml", "w") as fo:
        fo.write(
            "steps:\n- name: gcr.io/cloud-builders/gsutil\n  args:\n    - '-m'\n    - 'rsync'\n    - '-d'\n    - '-r'\n    - 'dags'\n    - '{}'".format(
                gcp_dag_folder
            )
        )
        gcp_image_names = []
        for docker_info in docker_image_info:
            gcp_image_name = docker_info[0]
            path = docker_info[1]
            gcp_image_names.append(gcp_image_name)
            fo.write(
                "\n- name: 'gcr.io/cloud-builders/docker'\n  args: [ 'build', '-t','{}', '-f', '{}/Dockerfile', '.' ]".format(
                    gcp_image_name, path
                )
            )
        fo.write("\nimages: [{}]".format(",".join(f"'{image}'" for image in gcp_image_names)))



from datetime import datetime

import requests
from airflow import DAG
from airflow.providers.cncf.kubernetes.operators.kubernetes_pod import KubernetesPodOperator

default_args = {"owner": ownertoreplace, "start_date": datetime.now()}


def callback_post(callback_type, context):
    requests.post(
        callbackhooktoreplace,
        json={
            "callback_type": callback_type,
            "task_id": context["task"].task_id,
            "run_id": context["run_id"],
            "id": dag_id,
        },
    )


def _on_success(context):
    callback_post("success", context)


def _on_failure(context):
    callback_post("failure", context)


dag = DAG(
    dag_id,
    schedule_interval=scheduletoreplace,
    default_args=default_args,
    catchup=False,
    on_success_callback=successtoreplace,
    on_failure_callback=failuretoreplace,
)

with dag:
    t1 = KubernetesPodOperator(
        task_id="kubernetes_pod",
        name="kubernetes_pod",
        namespace="default",
        image=imagetoreplace,
        cmds=["magniv-cli", "run", "filetoreplace", "functiontoreplace"],
        on_failure_callback=failuretoreplace,
        on_success_callback=successtoreplace,
    )



from magniv.export.airflow import export_to_airflow
from magniv.utils.utils import _get_tasks_json


def export(
    gcp=False,
    gcp_project_id=None,
    gcp_dag_folder=None,
    callback_hook=None,
    env_file_path=None,
):
    task_list = _get_tasks_json("./dump.json")
    export_to_airflow(
        task_list,
        gcp=gcp,
        gcp_project_id=gcp_project_id,
        gcp_dag_folder=gcp_dag_folder,
        callback_hook=callback_hook,
        env_file_path=env_file_path,
    )



from setuptools import setup, find_packages

try:
    with open("proj_description.md", "r", encoding="utf-8") as fh:
        long_description = fh.read()
except:
    long_description = ""

setup(
    name="magniv",
    description="Magniv Core Library",
    author_email="hello@magniv.io",
    url="https://www.magniv.io",
    project_urls={"Documentation": "https://docs.magniv.io",},
    long_description=long_description,
    long_description_content_type="text/markdown",
    version="0.1.36",
    py_modules=["magniv"],
    packages=find_packages(),
    install_requires=["Click", "docker"],
    entry_points={"console_scripts": ["magniv-cli = magniv.scripts.magniv:cli",],},
    include_package_data=True,
)
